@@ -18,6 +18,7 @@ from Bio import AlignIO, SeqIO
 from Bio.Align import MultipleSeqAlignment
 from Bio.Seq import Seq
 from Bio.SeqRecord import SeqRecord
+from packaging import version
 
 # Presto imports
 from presto.Defaults import default_muscle_exec, default_usearch_exec, \
@@ -33,6 +34,33 @@ default_evalue = 1e-5
 default_max_hits = 100
 default_max_memory = 3000
 
+def getMuscleVersion(exec=default_muscle_exec):
+    """
+    Gets the version of the Muscle executable
+
+    Arguments:
+      exec (str): the name or path to the Muscle executable.
+
+    Returns:
+      str: version number.
+    """
+    # Build commandline
+    cmd = [exec, '-version']
+
+    # Run
+    try:
+        stdout_str = check_output(cmd, stderr=STDOUT, shell=False, universal_newlines=True)
+    except CalledProcessError as e:
+        printError('Running command: %s\n%s' % (' '.join(cmd), e.output))
+
+    # Extract version number
+    # stdout_str='MUSCLE v3.8.425 by Robert C. Edgar\n'
+    # stdout_str='muscle 5.1.linux64 [12f0e2]\nBuilt Jan 13 2022 23:17:13\n\n'
+    version = stdout_str.split()[1]
+    version = re.sub('^v','',version)
+    version = re.sub('\.linux.*$','',version)
+
+    return version
 
 def runMuscle(seq_list, aligner_exec=default_muscle_exec):
     """
@@ -51,25 +79,48 @@ def runMuscle(seq_list, aligner_exec=default_muscle_exec):
         return align
 
     # Set MUSCLE command
-    cmd = [aligner_exec, '-diags', '-maxiters', '2']
+    muscle_version = getMuscleVersion(aligner_exec)
+    if ( version.parse(muscle_version) >= version.parse('5.0') ):    
+      printError('Muscle5 is not supported. Detected muscle version ' + muscle_version)
+      # Commented out because very slow
+      # Muscle5 requires in and out .fasta
+      #in_handle  = tempfile.NamedTemporaryFile(suffix='.fasta', mode='w+t', encoding='utf-8')
+      #out_handle = tempfile.NamedTemporaryFile(suffix='.fasta', mode='w+t', encoding='utf-8')
 
-    # Convert sequences to FASTA and write to string
-    stdin_handle = StringIO()
-    SeqIO.write(seq_list, stdin_handle, 'fasta-2line')
-    stdin_str = stdin_handle.getvalue()
-    stdin_handle.close()
+      # Convert sequences to FASTA and write to file
+      #SeqIO.write(seq_list, in_handle.name, 'fasta-2line')  
+      #cmd = [aligner_exec, '-align', in_handle.name,'-output', out_handle.name]
 
-    # Open MUSCLE process
-    child = Popen(cmd, bufsize=-1, stdin=PIPE, stdout=PIPE, stderr=PIPE,
-                  universal_newlines=True)
+      # Open MUSCLE process
+      #child = Popen(cmd, bufsize=-1, stdin=PIPE, stdout=PIPE, stderr=PIPE,
+      #              universal_newlines=True)
+      
+      #stdout_str, __ = child.communicate()
 
-    # Send sequences to MUSCLE stdin and retrieve stdout, stderr
-    stdout_str, __ = child.communicate(stdin_str)
+      # Read sequences from MUSCLE output
+      #align = AlignIO.read(out_handle.name, 'fasta')
+      #out_handle.close()
+  
+    else:
+      cmd = [aligner_exec, '-diags', '-maxiters', '2']
 
-    # Capture sequences from MUSCLE stdout
-    stdout_handle = StringIO(stdout_str)
-    align = AlignIO.read(stdout_handle, 'fasta')
-    stdout_handle.close()
+      # Convert sequences to FASTA and write to string
+      stdin_handle = StringIO()
+      SeqIO.write(seq_list, stdin_handle, 'fasta-2line')
+      stdin_str = stdin_handle.getvalue()
+      stdin_handle.close()
+
+      # Open MUSCLE process
+      child = Popen(cmd, bufsize=-1, stdin=PIPE, stdout=PIPE, stderr=PIPE,
+                    universal_newlines=True)
+
+      # Send sequences to MUSCLE stdin and retrieve stdout, stderr
+      stdout_str, __ = child.communicate(stdin_str)
+
+      # Capture sequences from MUSCLE stdout
+      stdout_handle = StringIO(stdout_str)
+      align = AlignIO.read(stdout_handle, 'fasta')
+      stdout_handle.close()
 
     return align
 
@@ -315,7 +366,7 @@ def makeBlastnDb(ref_file, db_exec=default_blastdb_exec):
     SeqIO.write(ref_dict.values(), seq_handle, format='fasta-2line')
     seq_handle.seek(0)
 
-    # Define usearch command
+    # Define makeblastdb command
     cmd = [db_exec,
            '-in', seq_handle.name,
            '-out', os.path.join(db_handle.name, 'reference'),
@@ -325,9 +376,11 @@ def makeBlastnDb(ref_file, db_exec=default_blastdb_exec):
     try:
         stdout_str = check_output(cmd, stderr=STDOUT, shell=False,
                                   universal_newlines=True)
-    except:
+    except CalledProcessError as e:
         seq_handle.close()
-        printError('Failed to make blastn database.')
+        db_exec_error = e.output.splitlines()
+        error_message =  db_exec_error[len(db_exec_error)-1]
+        raise ValueError('Failed to make blastn database. ' + error_message)
 
     # Close temporary sequence file
     seq_handle.close()
